package no.uit.mar157.datagenerator;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.HBaseConfiguration;
import org.apache.hadoop.hbase.client.HTable;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.File;
import java.io.IOException;

/**
 * Created by Martin Ernstsen <martin.ernstsen@gmail.com>
 * Date: 10/21/13
 * Time: 9:26 PM
 */
public class DataLoaderApp {
    private static Logger logger = LogManager.getLogger(DataLoaderApp.class.getName());
    protected static final String TABLE_NAME = "mar157_mario";
    protected static final String COL_INPUT = "in";
    protected static final String CF_INPUT = "in";

    int numberOfSequences;
    private File tmpFastaFile;

    public DataLoaderApp(int numberOfSequences) {
        logger.info("Starting DataLoaderApp. Intention is to make {} sequences", numberOfSequences);
        this.numberOfSequences = numberOfSequences;
        tmpFastaFile = new File("testdata.fas");
    }

    private void makeFastaFile() {
        DataGenerator dg = new DataGenerator(tmpFastaFile, null);
        try {
            dg.makeFastaFile(numberOfSequences);
        } catch (IOException e) {
            logger.error("Error writing to fasta file");
        }
    }

    private void insertIntoHBase() {
        Configuration config = HBaseConfiguration.create();
        try (HTable table = new HTable(config, TABLE_NAME)) {
            FastaToHbaseLoader loader = new FastaToHbaseLoader(table, tmpFastaFile);
            loader.load();
        } catch (IOException e) {
            logger.error("Error opening HBase table");
        }
    }

    public static void main(String[] args) {
        int numberOfSequences = Integer.parseInt(args[0]);
        DataLoaderApp app = new DataLoaderApp(numberOfSequences);
            app.makeFastaFile();
            app.insertIntoHBase();
    }
}
