package no.uit.mar157.datagenerator;

import org.apache.hadoop.hbase.client.HTable;
import org.apache.hadoop.hbase.client.Put;
import org.apache.hadoop.hbase.util.Bytes;

import java.io.*;

public class FastaToHbaseLoader {
	private HTable table;
    private File inputFile;

    /**
     * Load sequences from a fasta file into hbase
     *
     * @param table     the hbase table
     * @param inputFile the fasta file
     */
    public FastaToHbaseLoader(HTable table, File inputFile) {
        this.table = table;
        this.inputFile = inputFile;
    }
	
	/**
	 * @param entry the entry to load into Hbase
	 */
	private void hbasePut(FastaEntry entry) throws IOException {
		Put p = new Put(Bytes.toBytes(entry.getDescription()));
        p.add(Bytes.toBytes(DataLoaderApp.CF_INPUT), Bytes.toBytes(DataLoaderApp.COL_INPUT),
                Bytes.toBytes(entry.getEntry()));
        this.table.put(p);
	}
	
	protected void load() throws IOException {
		int entryCount = 0;
		boolean isInsideSequence = false;
		FastaEntry entry = null;

		try (BufferedReader bw = new BufferedReader(new FileReader(this.inputFile))) {
            String line;
			while ((line = bw.readLine()) != null) {
				if (line.startsWith(">")) {
					//Start of new entry.
					if (isInsideSequence) {
						// Previous entry complete. Send to Hbase.
						hbasePut(entry);
					}
					entry = new FastaEntry();
					entryCount++;
					String[] lineTokens = line.split(" +");
					String description = lineTokens[0];
					entry.setDescription(description);
					isInsideSequence = true;
				} else if (isInsideSequence) {
					entry.addSequenceLine(line);
				}
			}
			if (isInsideSequence) {
				//Send last entry to Hbase
				hbasePut(entry);
			}
			System.out.println("Total number of sequences inserted: " + Integer.toString(entryCount));
	    }
    }
}
