package no.uit.mar157.master;

import no.uit.mar157.common.PipelineDescription;
import no.uit.mar157.common.PipelineStage;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.sql.*;
import java.util.ArrayList;
import java.util.Set;

/**
 * Created by Martin Ernstsen <martin.ernstsen@gmail.com>
 * Date: 10/15/13
 * Time: 2:28 PM
 */
public class VersionManager {
    private static Logger logger = LogManager.getLogger(Master.class.getName());
    private PipelineDescription pipelineDescription;
    final String DATABASE_URL = "jdbc:mysql://localhost/mario";
    Connection connection = null;
    Statement statement = null;
    ResultSet resultSet = null;

    ArrayList<Long> equivalentVersions;
    ArrayList<Set<Long>> newEquivalentVersions;
    ArrayList<Long> currentVersions;

    PreparedStatement insertStageVersion;
    PreparedStatement getSimilarStage;
    PreparedStatement getMaxSequenceNumberForStage;
    PreparedStatement getSequenceNumberForId;
    PreparedStatement incrementCurrentVersion;
    PreparedStatement updateCurrentVersion;
    PreparedStatement getCurrentVersion;
    PreparedStatement getAllCurrentVersions;

    public void init(boolean startWithEmptyTables) throws SQLException {
        pipelineDescription = null;
        connection = null;
        statement = null;
        resultSet = null;
        equivalentVersions = null;
        newEquivalentVersions = null;
        currentVersions = null;
        connection = DriverManager.getConnection(DATABASE_URL, "mario", "123qwe");
        insertStageVersion = connection.prepareStatement(
                "INSERT INTO stageversion " +
                        "(stage, parent, sequence_number, hash, data) " +
                        "VALUES (?, ?, ?, ?, ?)",
                Statement.RETURN_GENERATED_KEYS
        );
        getSimilarStage = connection.prepareStatement(
                "SELECT id, data FROM stageversion " +
                "WHERE stage = ? AND parent = ? AND hash = ?"
        );
        getMaxSequenceNumberForStage = connection.prepareStatement(
                "SELECT MAX(sequence_number) FROM stageversion " +
                "WHERE stage = ?"
        );
        getSequenceNumberForId = connection.prepareStatement(
                "SELECT sequence_number FROM stageversion " +
                "WHERE id = ?"
        );
        incrementCurrentVersion = connection.prepareStatement(
                "UPDATE stage SET current_version = current_version + 1 " +
                "WHERE id = ?"
        );
        updateCurrentVersion = connection.prepareStatement(
                "UPDATE stage SET current_version = ? " +
                "WHERE id = ?"
        );
        getCurrentVersion = connection.prepareStatement(
                "SELECT current_version FROM stage " +
                "WHERE id = ?"
        );
        getAllCurrentVersions = connection.prepareStatement(
                "SELECT current_version FROM stage"
        );
        if (startWithEmptyTables) {
            logger.debug("Dropping stage and stageversion tables");
            statement = connection.createStatement();
            String droptablesQuery = "DROP TABLE IF EXISTS stageversion, stage";
            statement.execute(droptablesQuery);

            statement = connection.createStatement();
            String createStageTableQuery = "create table stage (" +
                    "id INT UNSIGNED NOT NULL AUTO_INCREMENT," +
                    "name VARCHAR(255) NOT NULL DEFAULT \"\"," +
                    "current_version INT UNSIGNED NOT NULL DEFAULT 1," +
                    "description VARCHAR(255) NOT NULL DEFAULT \"\"," +
                    "PRIMARY KEY(id)" +
                    ")";
            statement.execute(createStageTableQuery);

            statement = connection.createStatement();
            String createStageVersionQuery = "create table stageversion (" +
                    "id INT UNSIGNED NOT NULL AUTO_INCREMENT," +
                    "stage INT UNSIGNED NOT NULL REFERENCES stage(id)," +
                    "parent INT UNSIGNED REFERENCES stageversion(id)," +
                    "sequence_number INT UNSIGNED NOT NULL," +
                    "hash INT NOT NULL," +
                    "data TEXT NOT NULL," +
                    "PRIMARY KEY(id)" +
                    ")";
            statement.execute(createStageVersionQuery);
        }
    }

    public void update(PipelineDescription newPipelineDescription) throws SQLException {
        logger.debug("VersionManager updating description");

        //For each newstage in newPipelinedescription:
        //  check if newstage is similar to old stage in stageversion table
        //  if similar:
        //    update current_version in stage table to the value of the similar version
        //  else:
        //    find next version number for stage
        //    insert stage in stageversion table with new version number
        //    update current version in stage table with new version number

        boolean isFirstUpdate = false;
        if (pipelineDescription == null) {
            isFirstUpdate = true;
            createStageTableEntries(newPipelineDescription.size());
        }

        Long parent = 0L;
        for (int i = 0; i < newPipelineDescription.size(); i++) {
            PipelineStage stage = newPipelineDescription.getStage(i);

            if (isFirstUpdate) {
                updateCurrentVersion(i + 1, 1L);
                parent = insertStageVersion(i + 1, stage, parent, 1L);
            } else {
                Long similar = containsSimilarStage(i + 1, parent, stage);
                logger.debug("similar: stageNumber: {} parent: {}", i+1, parent);
                if (similar > 0L) {
                    getSequenceNumberForId.setLong(1, similar);
                    resultSet = getSequenceNumberForId.executeQuery();
                    resultSet.next();
                    Long seqNo = resultSet.getLong("sequence_number");
                    updateCurrentVersion(i + 1, seqNo);
                    parent = similar;
                } else {
                    getMaxSequenceNumberForStage.setInt(1, i + 1);
                    resultSet = getMaxSequenceNumberForStage.executeQuery();
                    resultSet.next();
                    Long newSequenceNumber = resultSet.getLong("MAX(sequence_number)") + 1L;
                    updateCurrentVersion(i + 1, newSequenceNumber);
                    parent = insertStageVersion(i + 1, stage, parent, newSequenceNumber);
                }
            }
        }
        pipelineDescription = newPipelineDescription;
        updateCurrentVersionsField();
        pipelineDescription.setCurrentVersion(currentVersions); // TODO cleanup
        logger.debug("pipelinedescription is now: {}", pipelineDescription);
        logger.debug("currentVersion is now: {}", currentVersions);
    }

    private Long insertStageVersion(int stageNumber, PipelineStage stage, Long parent, Long newSequenceNumber) throws SQLException {
        Long id = 0L;
        insertStageVersion.setInt(1, stageNumber);
        insertStageVersion.setLong(2, parent);
        insertStageVersion.setLong(3, newSequenceNumber);
        insertStageVersion.setInt(4, stage.toString().hashCode());
        insertStageVersion.setString(5, stage.toString());
        insertStageVersion.executeUpdate();
        resultSet = insertStageVersion.getGeneratedKeys();
        if (resultSet.next()) {
            id = resultSet.getLong(1);
        } else {
            logger.error("Error inserting stage");
        }
        return id;
    }

    private void updateCurrentVersion(int stageNumber, Long newVersion) throws SQLException {
        updateCurrentVersion.setLong(1, newVersion);
        updateCurrentVersion.setInt(2, stageNumber);
//        logger.debug("updating current version: {}", updateCurrentVersion.toString());
        updateCurrentVersion.executeUpdate();
    }

    private void updateCurrentVersionsField() throws SQLException {
        resultSet = getAllCurrentVersions.executeQuery();
        ArrayList<Long> cv = new ArrayList<>();
        while (resultSet.next()) {
            Long value = resultSet.getLong("current_version");
            cv.add(value);
        }
        currentVersions = cv;
    }

    private void createStageTableEntries(int numberOfStages) throws SQLException {
        logger.debug("Creating entries in stage table");
        statement = connection.createStatement();
        for (int i = 0; i < numberOfStages; i++) {
            String stageName = "\"stage" + String.valueOf(i+1) + "\"";
            statement.execute( "INSERT INTO stage (name) VALUE (" + stageName + ")" );
        }
    }

    private Long containsSimilarStage(int stageNumber, Long parent, PipelineStage stage) throws SQLException {
        Long retVal = 0L;
        getSimilarStage.setInt(1, stageNumber);
        getSimilarStage.setLong(2, parent);
        getSimilarStage.setInt(3, stage.toString().hashCode());
        resultSet = getSimilarStage.executeQuery();
        while (resultSet.next()) {
            // If we get here, the stage number and the hash are equal to some stage in the DB. Must now compare
            // data to be sure it is the same thing, and not caused by hash collision.
            if (stage.toString().equals(resultSet.getString("data"))) {
                retVal = resultSet.getLong("id");
                break;
            }
        }
        return retVal;
    }

    public PipelineDescription getPipelineDescription() {
        return pipelineDescription;
    }

    public ArrayList<Long> getCurrentVersions() {
        return currentVersions;
    }

    public void close() {
        try {
            if (connection != null) connection.close();
            if (statement != null) statement.close();
            if (resultSet != null) resultSet.close();
        } catch (SQLException e) {
            logger.error("Error closing DB connection {}", e.getMessage());
        }
    }
}
